import { of, throwError } from 'rxjs';
import { __extends, __spread, __values, __decorate, __metadata } from 'tslib';
import { delay } from 'rxjs/operators';
import { Injectable, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { SmdbApiService } from '@frontendart/smdb-api';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Apply this decorator, to keep track of anything that your classes methods return. A console.debug is applied to returned values.
 * If a method returns an Observable, a subscription is made, and the resolved data is logged instead of logging the Observable itself.
 *
 * @param {?=} excludes specifies methods (by name) to exclude from logging. By default the constructor is excluded
 * @return {?}
 */
function MethodLoggerDecorator(excludes) {
    if (excludes === void 0) { excludes = ['constructor']; }
    return (/**
     * @template T
     * @param {?} originalConstructor
     * @return {?}
     */
    function (originalConstructor) {
        return /** @class */ (function (_super) {
            __extends(class_1, _super);
            function class_1() {
                var params = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    params[_i] = arguments[_i];
                }
                var _this = _super.apply(this, __spread(params)) || this;
                console.debug("Overriding all functions of class " + originalConstructor.name + " to log method results!");
                var _loop_1 = function (prop) {
                    /** @type {?} */
                    var name_1 = (/** @type {?} */ (prop));
                    if (typeof this_1[prop] === 'function' && excludes.indexOf(prop) === -1) {
                        /** @type {?} */
                        var originalMethod_1 = (/** @type {?} */ (((/** @type {?} */ (this_1[prop])))));
                        this_1[name_1] = (/**
                         * @param {...?} args
                         * @return {?}
                         */
                        function () {
                            var args = [];
                            for (var _i = 0; _i < arguments.length; _i++) {
                                args[_i] = arguments[_i];
                            }
                            /** @type {?} */
                            var ret = originalMethod_1.apply(this, args);
                            if (ret.pipe) {
                                /** @type {?} */
                                var obs = (/** @type {?} */ (ret));
                                obs.subscribe((/**
                                 * @param {?} d
                                 * @return {?}
                                 */
                                function (d) {
                                    console.debug(originalConstructor.name + "." + prop + "(", args, ") resolved to:", d);
                                }));
                            }
                            else {
                                console.debug(originalConstructor.name + "." + prop + "(", args, ") returned with:", ret);
                            }
                            return ret;
                        });
                    }
                };
                var this_1 = this;
                // tslint:disable-next-line: forin
                for (var prop in _this) {
                    _loop_1(prop);
                }
                return _this;
            }
            return class_1;
        }(originalConstructor));
    });
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var DelayService = /** @class */ (function () {
    function DelayService() {
        this.delay = 5;
    }
    /**
     * @param {?} newDelay
     * @return {?}
     */
    DelayService.prototype.changeDelay = /**
     * @param {?} newDelay
     * @return {?}
     */
    function (newDelay) {
        this.delay = newDelay;
    };
    /**
     * @return {?}
     */
    DelayService.prototype.getDelay = /**
     * @return {?}
     */
    function () {
        return delay(new Date(Date.now() + Math.random() * 1000 * this.delay));
    };
    DelayService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    DelayService.ctorParameters = function () { return []; };
    return DelayService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
var SmdbApiMockService = /** @class */ (function () {
    function SmdbApiMockService(delayService) {
        this.delayService = delayService;
    }
    SmdbApiMockService_1 = SmdbApiMockService;
    /**
     * @param {?} token
     * @return {?}
     */
    SmdbApiMockService.prototype.checkToken = /**
     * @param {?} token
     * @return {?}
     */
    function (token) {
        if (token && token.length > 10) {
            /** @type {?} */
            var tokenDetails = {
                token: token,
                queriesLeft: Math.floor(Math.random() * 100),
                subscriptionType: 'PRO',
                uuid: '342342342ohu2h34ui23h42i3u4',
                valid: true,
            };
            return of(tokenDetails).pipe(this.delayService.getDelay());
        }
        else {
            return of({
                token: token,
                valid: false,
            })
                .pipe(this.delayService.getDelay())
                .pipe((/**
             * @param {?} data
             * @return {?}
             */
            function (data) { return throwError('Invalid token'); }));
        }
    };
    /**
     * @param {?} branchId
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getDeveloperStatistics = /**
     * @param {?} branchId
     * @param {?} tokenDetails
     * @return {?}
     */
    function (branchId, tokenDetails) {
        /** @type {?} */
        var ret = [];
        for (var i = 0; i < Math.floor(Math.random() * 40) + 7; i++) {
            /** @type {?} */
            var elem = {
                name: 'Developer' + Math.floor(Math.random() * 40) + 1,
                developerStat: {
                    averageAddedLines: Math.floor(Math.random() * 100),
                    averageModifiedLines: Math.floor(Math.random() * 100),
                    averageRemovedLines: Math.floor(Math.random() * 100),
                    averageRuleViolationsCreated: Math.floor(Math.random() * 100),
                    averageRuleViolationsRemoved: Math.floor(Math.random() * 100),
                    maintainabilityChange: Math.floor(Math.random() * 100),
                    numberOfCommits: Math.floor(Math.random() * 100),
                    totalAddedLines: Math.floor(Math.random() * 100),
                    totalModifiedLines: Math.floor(Math.random() * 100),
                    totalRemovedLines: Math.floor(Math.random() * 100),
                    totalRuleViolationsCreated: Math.floor(Math.random() * 100),
                    totalRuleViolationsRemoved: Math.floor(Math.random() * 100),
                },
            };
            ret.push(elem);
        }
        return of(ret).pipe(delay(new Date(Date.now() + Math.random() * 3000)));
    };
    /**
     * @param {?} branch
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getBranchMetric = /**
     * @param {?} branch
     * @param {?} tokenDetails
     * @return {?}
     */
    function (branch, tokenDetails) {
        /**
         * @param {?} length
         * @return {?}
         */
        function makeid(length) {
            /** @type {?} */
            var result = '';
            /** @type {?} */
            var characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
            /** @type {?} */
            var charactersLength = characters.length;
            for (var i = 0; i < length; i++) {
                result += characters.charAt(Math.floor(Math.random() * charactersLength));
            }
            return result;
        }
        return of([
            {
                description: 'Total Logical Lines of Code',
                longDescription: '<b>Method:</b> number of non-empty and non-comment code lines of the method, including the non-empty and non-comment lines of its anonymous and local classes.<br><b>Class:</b> number of non-empty and non-comment code lines of the class, including the non-empty and non-comment code lines of its local methods, anonymous, local, and nested classes.<br><b>Package:</b> number of non-empty and non-comment code lines of the package, including its subpackages.<br><b>Component:</b> number of non-empty and non-comment code lines of the component, including its subcomponents.',
                metricDefinitionId: 656,
                name: 'TLLOC',
                package: 'SourceMeter',
                sourceCodeElementTypes: [],
            },
            {
                package: 'SourceMeter',
                metricDefinitionId: 537,
                name: 'TNCL',
                description: 'Total Number of Classes',
                longDescription: '<b>Package:</b> number of classes in the package, including the classes of its subpackages.<br><b>Component:</b> number of classes in the component, including the classes of its subcomponents.',
                sourceCodeElementTypes: [],
            },
            {
                name: 'TNPKG',
                description: 'Total Number of Packages',
                package: 'SourceMeter',
                metricDefinitionId: 533,
                longDescription: '<b>Package:</b> number of subpackages in the package, including all directly or indirectly contained subpackages.<br><b>Component:</b> number of packages and subpackages that belong to the component, including its subcomponents.',
                sourceCodeElementTypes: [],
            },
            {
                name: 'TNM',
                description: 'Total Number of Methods',
                package: 'SourceMeter',
                metricDefinitionId: 616,
                longDescription: '<b>Class:</b> number of methods in the class, including the inherited ones, as well as the inherited and local methods of its nested, anonymous, and local classes. Methods that override abstract methods are not counted.<br><b>Package:</b> number of methods in the package, including the methods of its subpackages.<br><b>Component:</b> number of methods in the component, including the methods of its subcomponents.',
                sourceCodeElementTypes: [],
            },
            {
                name: 'CC',
                description: 'Clone Coverage',
                package: 'SourceMeter',
                longDescription: '<b>Method/class/package:</b> ratio of code covered by code duplications in the source code element to the size of the source code element, expressed in terms of the number of syntactic entities (statements, expressions, etc.).<br><b>Component:</b> ratio of code covered by code duplications in the component to the size of the component, expressed in terms of the number of syntactic entities (statements, expressions, etc.).',
                metricDefinitionId: 138,
                sourceCodeElementTypes: [],
            },
            {
                description: 'Clone Classes',
                name: 'CCL',
                package: 'SourceMeter',
                longDescription: '<b>Method/class/package:</b> number of clone classes having at least one clone instance in the source code element.<br><b>Component:</b> number of clone classes having at least one clone instance in the component.',
                metricDefinitionId: 490,
                sourceCodeElementTypes: [],
            },
            {
                description: 'Clone Instances',
                name: 'CI',
                package: 'SourceMeter',
                longDescription: '<b>Method/class/package:</b> number of clone instances in the source code element.<br><b>Component:</b> number of clone instances in the component.<br><b>Clone class:</b> number of clone instances in the clone class.',
                metricDefinitionId: 143,
                sourceCodeElementTypes: [],
            },
            {
                description: 'Minor Warnings',
                name: 'WarningMinor',
                package: 'SourceMeter',
                longDescription: 'Number of warnings with Minor priority',
                metricDefinitionId: 607,
                sourceCodeElementTypes: [],
            },
            {
                description: 'Blocker Warnings',
                name: 'WarningBlocker',
                package: 'SourceMeter',
                longDescription: 'Number of warnings with Blocker priority',
                metricDefinitionId: 164,
                sourceCodeElementTypes: [],
            },
            {
                description: 'Info Warnings',
                name: 'WarningInfo',
                package: 'SourceMeter',
                longDescription: 'Number of warnings with Info priority',
                metricDefinitionId: 596,
                sourceCodeElementTypes: [],
            },
            {
                description: 'Critical Warnings',
                name: 'WarningCritical',
                package: 'SourceMeter',
                longDescription: 'Number of warnings with Critical priority',
                metricDefinitionId: 433,
                sourceCodeElementTypes: [],
            },
            {
                description: 'Major Warnings',
                name: 'WarningMajor',
                package: 'SourceMeter',
                longDescription: 'Number of warnings with Major priority',
                metricDefinitionId: 404,
                sourceCodeElementTypes: [],
            },
            {
                description: 'Total Number of Methods',
                name: 'TNPKG',
                package: 'SourceMeter3',
                longDescription: makeid(15),
                metricDefinitionId: this.generateId(),
                sourceCodeElementTypes: [],
            },
            {
                description: 'Total of Methods',
                name: 'TNM',
                package: 'SourceMeter2',
                longDescription: makeid(15),
                metricDefinitionId: this.generateId(),
                sourceCodeElementTypes: [],
            },
            {
                description: makeid(15),
                name: 'CC',
                package: 'SourceMeter2',
                longDescription: makeid(15),
                metricDefinitionId: this.generateId(),
                sourceCodeElementTypes: [],
            },
            {
                description: 'Total Number',
                name: makeid(4),
                package: 'SourceMeter2',
                longDescription: makeid(15),
                metricDefinitionId: this.generateId(),
                sourceCodeElementTypes: [],
            },
        ]).pipe(this.delayService.getDelay());
    };
    /**
     * @param {?} branch
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getBranchDetails = /**
     * @param {?} branch
     * @param {?} tokenDetails
     * @return {?}
     */
    function (branch, tokenDetails) {
        /** @type {?} */
        var foundIndex = SmdbApiMockService_1._BRANCHES.findIndex((/**
         * @param {?} value
         * @return {?}
         */
        function (value) { return value.id === branch; }));
        console.debug('Branch sample found: ' + foundIndex);
        /** @type {?} */
        var returnData = foundIndex !== -1
            ? SmdbApiMockService_1._BRANCHES[foundIndex]
            : SmdbApiMockService_1.SAMPLE_BRANCH;
        return of(returnData).pipe(this.delayService.getDelay());
    };
    /**
     * @param {?} versionId
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getVersionDetails = /**
     * @param {?} versionId
     * @param {?} tokenDetails
     * @return {?}
     */
    function (versionId, tokenDetails) {
        return of({
            date: '2011-09-20T02:22:57.000+0000',
            hash: '097102081ab68bdefcbc8ad7977a760565f87e51',
            committer: 'Charles Fry',
            projectName: 'google_guava',
            url: 'https://github.com/google/guava.git',
            log: 'Sample log message',
            branches: ['master', 'develop'],
            language: 'Java',
            metrics: [
                {
                    package: 'SourceMeter',
                    metricId: 656,
                    name: 'TLLOC',
                    description: 'Total Logical Lines of Code',
                    displayname: 'Total Logical Lines of Code',
                    // tslint:disable-next-line:max-line-length
                    longDescription: '<b>Method:</b> number of non-empty and non-comment code lines of the method, including the non-empty and non-comment lines of its anonymous and local classes.<br><b>Class:</b> number of non-empty and non-comment code lines of the class, including the non-empty and non-comment code lines of its local methods, anonymous, local, and nested classes.<br><b>Package:</b> number of non-empty and non-comment code lines of the package, including its subpackages.<br><b>Component:</b> number of non-empty and non-comment code lines of the component, including its subcomponents.',
                    value: 128581,
                    prevValue: 128581,
                },
                {
                    package: 'SourceMeter',
                    metricId: 537,
                    name: 'TNCL',
                    description: 'Total Number of Classes',
                    displayname: 'Total Number of Classes',
                    // tslint:disable-next-line:max-line-length
                    longDescription: '<b>Package:</b> number of classes in the package, including the classes of its subpackages.<br><b>Component:</b> number of classes in the component, including the classes of its subcomponents.',
                    value: 2916,
                    prevValue: 128581,
                },
                {
                    package: 'SourceMeter',
                    metricId: 533,
                    name: 'TNPKG',
                    description: 'Total Number of Packages',
                    displayname: 'Total Number of Packages',
                    // tslint:disable-next-line:max-line-length
                    longDescription: '<b>Package:</b> number of subpackages in the package, including all directly or indirectly contained subpackages.<br><b>Component:</b> number of packages and subpackages that belong to the component, including its subcomponents.',
                    value: 24,
                    prevValue: 128581,
                },
                {
                    package: 'SourceMeter',
                    metricId: 616,
                    name: 'TNM',
                    description: 'Total Number of Methods',
                    displayname: 'Total Number of Methods',
                    // tslint:disable-next-line:max-line-length
                    longDescription: '<b>Class:</b> number of methods in the class, including the inherited ones, as well as the inherited and local methods of its nested, anonymous, and local classes. Methods that override abstract methods are not counted.<br><b>Package:</b> number of methods in the package, including the methods of its subpackages.<br><b>Component:</b> number of methods in the component, including the methods of its subcomponents.',
                    value: 16493,
                    prevValue: 128581,
                },
                {
                    package: 'SourceMeter',
                    metricId: 138,
                    name: 'CC',
                    description: 'Clone Coverage',
                    displayname: 'Clone Coverage',
                    // tslint:disable-next-line:max-line-length
                    longDescription: '<b>Method/class/package:</b> ratio of code covered by code duplications in the source code element to the size of the source code element, expressed in terms of the number of syntactic entities (statements, expressions, etc.).<br><b>Component:</b> ratio of code covered by code duplications in the component to the size of the component, expressed in terms of the number of syntactic entities (statements, expressions, etc.).',
                    value: 0.1966537,
                    prevValue: 128581,
                },
                {
                    package: 'SourceMeter',
                    metricId: 490,
                    name: 'CCL',
                    description: 'Clone Classes',
                    displayname: 'Clone Classes',
                    // tslint:disable-next-line:max-line-length
                    longDescription: '<b>Method/class/package:</b> number of clone classes having at least one clone instance in the source code element.<br><b>Component:</b> number of clone classes having at least one clone instance in the component.',
                    value: 715,
                    prevValue: 128581,
                },
                {
                    package: 'SourceMeter',
                    metricId: 143,
                    name: 'CI',
                    description: 'Clone Instances',
                    displayname: 'Clone Instances',
                    // tslint:disable-next-line:max-line-length
                    longDescription: '<b>Method/class/package:</b> number of clone instances in the source code element.<br><b>Component:</b> number of clone instances in the component.<br><b>Clone class:</b> number of clone instances in the clone class.',
                    value: 1868,
                    prevValue: 128581,
                },
                {
                    package: 'SourceMeter',
                    metricId: 607,
                    name: 'WarningMinor',
                    description: 'Minor Warnings',
                    displayname: 'Minor Warnings',
                    longDescription: 'Number of warnings with Minor priority',
                    value: 15730,
                    prevValue: 128581,
                },
                {
                    package: 'SourceMeter',
                    metricId: 164,
                    name: 'WarningBlocker',
                    description: 'Blocker Warnings',
                    displayname: 'Blocker Warnings',
                    longDescription: 'Number of warnings with Blocker priority',
                    value: 0,
                    prevValue: 128581,
                },
                {
                    package: 'SourceMeter',
                    metricId: 596,
                    name: 'WarningInfo',
                    description: 'Info Warnings',
                    displayname: 'Info Warnings',
                    longDescription: 'Number of warnings with Info priority',
                    value: 55583,
                    prevValue: 128581,
                },
                {
                    package: 'SourceMeter',
                    metricId: 433,
                    name: 'WarningCritical',
                    description: 'Critical Warnings',
                    displayname: 'Critical Warnings',
                    longDescription: 'Number of warnings with Critical priority',
                    value: 1453,
                    prevValue: 128581,
                },
                {
                    package: 'SourceMeter',
                    metricId: 404,
                    name: 'WarningMajor',
                    description: 'Major Warnings',
                    displayname: 'Major Warnings',
                    longDescription: 'Number of warnings with Major priority',
                    value: 2549,
                    prevValue: 128581,
                },
            ],
        }).pipe(this.delayService.getDelay());
    };
    /**
     * @param {?} projectName
     * @param {?} pageSize
     * @param {?} page
     * @param {?} privateOnly
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.findProjects = /**
     * @param {?} projectName
     * @param {?} pageSize
     * @param {?} page
     * @param {?} privateOnly
     * @param {?} tokenDetails
     * @return {?}
     */
    function (projectName, pageSize, page, privateOnly, tokenDetails) {
        /** @type {?} */
        var numberOfProjects = Math.floor(Math.random() * 100);
        /** @type {?} */
        var projects = [];
        for (var i = 0; i < numberOfProjects; i++) {
            projects.push({
                analysisRunning: Math.random() < 0.2,
                language: 'java',
                projectName: 'Sample project ' +
                    i +
                    (projectName && Math.random() < 0.1 ? '(__' + projectName + '__)' : ''),
                url: 'https://github.com/' + i + '_th_project',
                id: this.generateId(10000, 20000),
                deleted: false,
                private: !!this.generateId(0, 1),
            });
        }
        if (projectName) {
            projects = projects.filter((/**
             * @param {?} e
             * @return {?}
             */
            function (e) { return e.projectName.indexOf(projectName) !== -1; }));
        }
        return of({
            content: projects.slice(0, pageSize),
            totalElements: projects.length,
        }).pipe(this.delayService.getDelay());
    };
    /**
     * @param {?} projectId
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getBranches = /**
     * @param {?} projectId
     * @param {?} tokenDetails
     * @return {?}
     */
    function (projectId, tokenDetails) {
        if (this.providedSampleBranches !== undefined) {
            return of(this.providedSampleBranches).pipe(this.delayService.getDelay());
        }
        else {
            return of((/** @type {?} */ ([
                SmdbApiMockService_1._SAMPLE_BRANCH,
                SmdbApiMockService_1._SAMPLE_BRANCH2,
                SmdbApiMockService_1._SAMPLE_BRANCH3,
            ]))).pipe(this.delayService.getDelay());
        }
    };
    Object.defineProperty(SmdbApiMockService.prototype, "setProvidedSampleBranches", {
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            this.providedSampleBranches = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SmdbApiMockService, "SAMPLE_BRANCH", {
        get: /**
         * @return {?}
         */
        function () {
            return this._SAMPLE_BRANCH;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} projectName
     * @param {?} branchName
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getTimeline = /**
     * @param {?} projectName
     * @param {?} branchName
     * @param {?} tokenDetails
     * @return {?}
     */
    function (projectName, branchName, tokenDetails) {
        return of("<svg width=\"120px\" height=\"120px\" viewBox=\"0 0 120 120\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\">\n        <!-- Generator: Sketch 40.1 (33804) - http://www.bohemiancoding.com/sketch -->\n        <title>line-in-motion</title>\n        <desc>Created with Sketch.</desc>\n        <defs></defs>\n        <g id=\"Page-1\" stroke=\"none\" stroke-width=\"1\" fill=\"none\" fill-rule=\"evenodd\">\n            <g id=\"line-in-motion\" fill=\"#000000\">\n                <path d=\"M9,0 L11,0 L11,20 L9,20 L9,0 Z M34.1339746,0.839745962 L35.8660254,1.83974596 L25.8660254,19.160254 L24.1339746,18.160254 L34.1339746,0.839745962 Z M14.1339746,20.839746 L15.8660254,21.839746 L5.8660254,39.160254 L4.1339746,38.160254 L14.1339746,20.839746 Z M58.160254,4.1339746 L59.160254,5.8660254 L41.839746,15.8660254 L40.839746,14.1339746 L58.160254,4.1339746 Z M18.160254,44.1339746 L19.160254,45.8660254 L1.83974596,55.8660254 L0.839745962,54.1339746 L18.160254,44.1339746 Z M80,9 L80,11 L60,11 L60,9 L80,9 Z M20,69 L20,71 L0,71 L0,69 L20,69 Z M99.3205081,14 L98.3205081,15.7320508 L81,5.73205081 L82,4 L99.3205081,14 Z M19.3205081,94 L18.3205081,95.7320508 L1,85.7320508 L2,84 L19.3205081,94 Z M115.866025,18.160254 L114.133975,19.160254 L104.133975,1.83974596 L105.866025,0.839745962 L115.866025,18.160254 Z M15.8660254,118.160254 L14.1339746,119.160254 L4.1339746,101.839746 L5.8660254,100.839746 L15.8660254,118.160254 Z M38.160254,24.1339746 L39.160254,25.8660254 L21.839746,35.8660254 L20.839746,34.1339746 L38.160254,24.1339746 Z M60,29 L60,31 L40,31 L40,29 L60,29 Z M79.3205081,34 L78.3205081,35.7320508 L61,25.7320508 L62,24 L79.3205081,34 Z M95.8660254,38.160254 L94.1339746,39.160254 L84.1339746,21.839746 L85.8660254,20.839746 L95.8660254,38.160254 Z M111,40 L109,40 L109,20 L111,20 L111,40 Z M114.133975,40.839746 L115.866025,41.839746 L105.866025,59.160254 L104.133975,58.160254 L114.133975,40.839746 Z M40,49 L40,51 L20,51 L20,49 L40,49 Z M59.3205081,54 L58.3205081,55.7320508 L41,45.7320508 L42,44 L59.3205081,54 Z M75.8660254,58.160254 L74.1339746,59.160254 L64.1339746,41.839746 L65.8660254,40.839746 L75.8660254,58.160254 Z M91,60 L89,60 L89,40 L91,40 L91,60 Z M94.1339746,60.839746 L95.8660254,61.839746 L85.8660254,79.160254 L84.1339746,78.160254 L94.1339746,60.839746 Z M118.160254,64.1339746 L119.160254,65.8660254 L101.839746,75.8660254 L100.839746,74.1339746 L118.160254,64.1339746 Z M39.3205081,74 L38.3205081,75.7320508 L21,65.7320508 L22,64 L39.3205081,74 Z M55.8660254,78.160254 L54.1339746,79.160254 L44.1339746,61.839746 L45.8660254,60.839746 L55.8660254,78.160254 Z M71,80 L69,80 L69,60 L71,60 L71,80 Z M74.1339746,80.839746 L75.8660254,81.839746 L65.8660254,99.160254 L64.1339746,98.160254 L74.1339746,80.839746 Z M98.160254,84.1339746 L99.160254,85.8660254 L81.839746,95.8660254 L80.839746,94.1339746 L98.160254,84.1339746 Z M120,89 L120,91 L100,91 L100,89 L120,89 Z M35.8660254,98.160254 L34.1339746,99.160254 L24.1339746,81.839746 L25.8660254,80.839746 L35.8660254,98.160254 Z M51,100 L49,100 L49,80 L51,80 L51,100 Z M54.1339746,100.839746 L55.8660254,101.839746 L45.8660254,119.160254 L44.1339746,118.160254 L54.1339746,100.839746 Z M78.160254,104.133975 L79.160254,105.866025 L61.839746,115.866025 L60.839746,114.133975 L78.160254,104.133975 Z M100,109 L100,111 L80,111 L80,109 L100,109 Z M119.320508,114 L118.320508,115.732051 L101,105.732051 L102,104 L119.320508,114 Z M31,120 L29,120 L29,100 L31,100 L31,120 Z\" id=\"Combined-Shape\"></path>\n            </g>\n        </g>\n        </svg>\n        ");
    };
    /**
     * @param {?} branch
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getBranchVersionData = /**
     * @param {?} branch
     * @param {?} tokenDetails
     * @return {?}
     */
    function (branch, tokenDetails) {
        return of((/** @type {?} */ ([
            {
                author: 'John Doe',
                committed: '2019-04-30T14:45:47.048Z',
                hash: '1234567891234556345353',
                id: 48,
                log: 'Sample commit log message a long one',
                parentHash: '21534544354324234234234324',
            },
            {
                author: 'Jane Doe',
                committed: '2018-04-30T14:45:47.048Z',
                hash: '123456fsdfsdf91234556345353',
                id: 15,
                log: 'Another Sample commit log message',
                parentHash: '2143434544354324234234234324',
            },
            {
                author: 'John Doe',
                committed: '2019-03-30T14:45:47.048Z',
                hash: '1234567891234556345353',
                id: 16,
                log: 'Sample commit log message a long one',
                parentHash: '21534544354324234234234324',
            },
            {
                author: 'Jane Doe',
                committed: '2018-05-01T14:45:47.048Z',
                hash: '123456fsdfsdf91234556345353',
                id: 18,
                log: 'Another Sample commit log message',
                parentHash: '2143434544354324234234234324',
            },
        ]))).pipe(this.delayService.getDelay());
    };
    /**
     * @param {?} type
     * @param {?} id
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getQualifications = /**
     * @param {?} type
     * @param {?} id
     * @param {?} tokenDetails
     * @return {?}
     */
    function (type, id, tokenDetails) {
        /** @type {?} */
        var numVersions = type === 'version' ? 1 : Math.floor(Math.random() * 20) + 10;
        /** @type {?} */
        var ret = [];
        /** @type {?} */
        var nextVal = (/**
         * @param {?=} prevVal
         * @return {?}
         */
        function (prevVal) {
            if (prevVal === void 0) { prevVal = Math.random() * 5; }
            if (Math.random() < 0.2) {
                prevVal += Math.random() * 5 - 2;
            }
            return prevVal + Math.random() * 2 - 1;
        });
        for (var i = 0; i < numVersions; i++) {
            /** @type {?} */
            var prevVal = i > 0 ? ret[i - 1].qualifications : {};
            /** @type {?} */
            var elem = {
                qualifications: {
                    'Code complexity': nextVal(prevVal['Code complexity']),
                    'Code documentation': nextVal(prevVal['Code documentation']),
                    'Code duplication': nextVal(prevVal['Code duplication']),
                    'Code size': nextVal(prevVal['Code size']),
                    'Code structure': nextVal(prevVal['Code structure']),
                    'Coding issues': nextVal(prevVal['Coding issues']),
                    Maintainability: nextVal(prevVal.Maintainability),
                },
                version: {
                    author: 'Author ' + (i % 10),
                    committed: new Date(new Date().getTime() + i * 10000000).toISOString(),
                    hash: '34543543534543534534534',
                    id: i,
                    log: 'Sample commit log message ' + i,
                    parentHash: '534543534543534534535',
                },
            };
            ret.push(elem);
        }
        return of(ret).pipe(this.delayService.getDelay());
    };
    /**
     * Mocks get /api/version/{versionId}/sourceCodeElementsByMetric future version
     */
    /**
     * Mocks get /api/version/{versionId}/sourceCodeElementsByMetric future version
     * @param {?} versionId
     * @param {?} params
     * @param {?=} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getSourceCodeElementsWithMetrics = /**
     * Mocks get /api/version/{versionId}/sourceCodeElementsByMetric future version
     * @param {?} versionId
     * @param {?} params
     * @param {?=} tokenDetails
     * @return {?}
     */
    function (versionId, params, tokenDetails) {
        /** @type {?} */
        var generateValues = (/**
         * @param {?} metricValue
         * @return {?}
         */
        function (metricValue) {
            /** @type {?} */
            var values = [];
            values.push({
                package: 'SourceMeter',
                metricId: 656,
                name: 'TLLOC',
                description: 'Total Logical Lines of Code',
                displayname: 'Total Logical Lines of Code',
                longDescription: '<b>Method:</b> number of non-empty and non-comment code lines of the method, including the non-empty and non-comment lines of its anonymous and local classes.<br><b>Class:</b> number of non-empty and non-comment code lines of the class, including the non-empty and non-comment code lines of its local methods, anonymous, local, and nested classes.<br><b>Package:</b> number of non-empty and non-comment code lines of the package, including its subpackages.<br><b>Component:</b> number of non-empty and non-comment code lines of the component, including its subcomponents.',
                value: metricValue,
                prevValue: metricValue,
            }, {
                package: 'SourceMeter',
                metricId: 537,
                name: 'TNCL',
                description: 'Total Number of Classes',
                displayname: 'Total Number of Classes',
                longDescription: '<b>Package:</b> number of classes in the package, including the classes of its subpackages.<br><b>Component:</b> number of classes in the component, including the classes of its subcomponents.',
                value: metricValue,
                prevValue: metricValue,
            }, {
                package: 'SourceMeter',
                metricId: 533,
                name: 'TNPKG',
                description: 'Total Number of Packages',
                displayname: 'Total Number of Packages',
                longDescription: '<b>Package:</b> number of subpackages in the package, including all directly or indirectly contained subpackages.<br><b>Component:</b> number of packages and subpackages that belong to the component, including its subcomponents.',
                value: metricValue,
                prevValue: metricValue,
            }, {
                package: 'SourceMeter',
                metricId: 143,
                name: 'CI',
                description: 'Clone Instances',
                displayname: 'Clone Instances',
                // tslint:disable-next-line:max-line-length
                longDescription: '<b>Method/class/package:</b> number of clone instances in the source code element.<br><b>Component:</b> number of clone instances in the component.<br><b>Clone class:</b> number of clone instances in the clone class.',
                value: metricValue,
                prevValue: metricValue,
            }, {
                package: 'SourceMeter',
                metricId: 607,
                name: 'WarningMinor',
                description: 'Minor Warnings',
                displayname: 'Minor Warnings',
                longDescription: 'Number of warnings with Minor priority',
                value: metricValue,
                prevValue: metricValue,
            }, {
                package: 'SourceMeter',
                metricId: 164,
                name: 'WarningBlocker',
                description: 'Blocker Warnings',
                displayname: 'Blocker Warnings',
                longDescription: 'Number of warnings with Blocker priority',
                value: metricValue,
                prevValue: metricValue,
            });
            return values;
        });
        /** @type {?} */
        var ret = [
            {
                path: 'guava-gwt/src-super/com/google/common/collect/super/com/google/common/collect/AbstractBiMap.java',
                line: 1,
                col: 1,
                endLine: 384,
                endCol: 1,
                uniqueName: 'com.google.common.collect',
                type: 'Package',
                sourceCodeElementId: 6344,
                value: generateValues(9),
                isNew: true,
            },
            {
                path: 'guava-gwt/src-super/com/google/common/base/super/com/google/common/base/Platform.java',
                line: 1,
                col: 1,
                endLine: 50,
                endCol: 1,
                uniqueName: 'com.google.common.base',
                type: 'Package',
                sourceCodeElementId: 3271,
                isNew: true,
                value: generateValues(12),
            },
            {
                path: 'guava-gwt/src-super/com/google/common/primitives/super/com/google/common/primitives/Chars.java',
                line: 1,
                col: 1,
                endLine: 533,
                endCol: 1,
                uniqueName: 'com.google.common.primitives',
                type: 'Package',
                sourceCodeElementId: 3860,
                isNew: true,
                value: generateValues(8),
            },
            {
                path: 'guava-tests/test/com/google/common/util/concurrent/AbstractChainedListenableFutureTest.java',
                line: 1,
                col: 1,
                endLine: 105,
                endCol: 1,
                uniqueName: 'com.google.common.util.concurrent',
                type: 'Package',
                sourceCodeElementId: 4196,
                isNew: true,
                value: generateValues(14),
            },
            {
                path: 'guava-gwt/test/com/google/common/net/TestModuleEntryPoint.java',
                line: 1,
                col: 1,
                endLine: 31,
                endCol: 1,
                uniqueName: 'com.google.common.net',
                type: 'Package',
                sourceCodeElementId: 83988,
                isNew: true,
                value: generateValues(1),
            },
            {
                path: 'guava-bootstrap/src/java/util/concurrent/ExecutorService.java',
                line: 1,
                col: 1,
                endLine: 48,
                endCol: 1,
                uniqueName: 'java.util.concurrent',
                type: 'Package',
                sourceCodeElementId: 182576,
                isNew: true,
                value: generateValues(8),
            },
            {
                path: 'guava-tests/test/com/google/common/io/AppendableWriterTest.java',
                line: 1,
                col: 1,
                endLine: 129,
                endCol: 1,
                uniqueName: 'com.google.common.io',
                type: 'Package',
                sourceCodeElementId: 3532,
                isNew: true,
                value: generateValues(3),
            },
            {
                path: 'guava-tests/test/com/google/common/math/BigIntegerMathTest.java',
                line: 1,
                col: 1,
                endLine: 445,
                endCol: 1,
                uniqueName: 'com.google.common.math',
                type: 'Package',
                sourceCodeElementId: 4913616,
                isNew: true,
                value: generateValues(7),
            },
            {
                path: 'guava-tests/test/com/google/common/eventbus/AsyncEventBusTest.java',
                line: 1,
                col: 1,
                endLine: 84,
                endCol: 1,
                uniqueName: 'com.google.common.eventbus',
                type: 'Package',
                sourceCodeElementId: 985350,
                isNew: true,
                value: generateValues(14),
            },
            {
                path: 'guava-tests/test/com/google/common/cache/AbstractCacheTest.java',
                line: 1,
                col: 1,
                endLine: 236,
                endCol: 1,
                uniqueName: 'com.google.common.cache',
                type: 'Package',
                sourceCodeElementId: 931623,
                isNew: true,
                value: generateValues(76),
            },
        ];
        for (var i = 0; i < 93; i++) {
            ret.push({
                path: 'src/random/org/apache/commons/lang/enum/Broken1Enum.java',
                line: 66,
                col: 5,
                endLine: 66,
                endCol: 66,
                uniqueName: 'random.filler.asd' + i,
                type: 'Attribute',
                sourceCodeElementId: 5468563,
                isNew: true,
                value: generateValues(12),
            });
            for (var j = 0; j < 17; j++) {
                ret.push({
                    path: 'src/random/org/apache/commons/lang/enum/Broken1Enum.java',
                    line: 66,
                    col: 5,
                    endLine: 66,
                    endCol: 66,
                    uniqueName: 'random.filler.asd' + i + '.' + j,
                    type: 'Attribute',
                    sourceCodeElementId: 5468563,
                    isNew: true,
                    value: generateValues(93),
                });
            }
        }
        if (params.uniqueName && params.uniqueName.length > 0) {
            ret = ret.filter((/**
             * @param {?} e
             * @return {?}
             */
            function (e) { return e.uniqueName.toLowerCase().includes(params.uniqueName.toLowerCase()); }));
        }
        if (params.type && params.type.length > 0) {
            ret = ret.filter((/**
             * @param {?} e
             * @return {?}
             */
            function (e) { return e.type.toLowerCase().includes(params.type.toLowerCase()); }));
        }
        /** @type {?} */
        var rows = ret.length;
        if (params.sort && params.sort.length > 0) {
            /** @type {?} */
            var dir = params.sort === 'asc';
            /** @type {?} */
            var factor_1 = dir ? 1 : -1;
            /** @type {?} */
            var filteredMetric = ret[0].value.filter((/**
             * @param {?} metric
             * @return {?}
             */
            function (metric) { return metric.metricId === params.metricDefinitionId; }));
            /** @type {?} */
            var i_1 = ret[0].value.indexOf(filteredMetric[0]);
            ret = ret.sort((/**
             * @param {?} a
             * @param {?} b
             * @return {?}
             */
            function (a, b) { return factor_1 * Math.sign(b.value[i_1].value - a.value[i_1].value); }));
        }
        /** @type {?} */
        var startIndex = params.size * params.page;
        /** @type {?} */
        var endIndex = params.size * (params.page + 1);
        return of({
            content: ret.slice(startIndex, endIndex),
            pageable: {
                sort: {
                    sorted: false,
                    unsorted: true,
                },
                offset: 0,
                pageSize: 10,
                pageNumber: 0,
                paged: true,
                unpaged: false,
            },
            totalPages: 2,
            totalElements: rows,
            last: false,
            number: 0,
            size: 10,
            sort: {
                sorted: false,
                unsorted: true,
            },
            first: true,
            numberOfElements: 10,
        }).pipe(this.delayService.getDelay());
    };
    /**
     * @private
     * @param {?=} min
     * @param {?=} max
     * @return {?}
     */
    SmdbApiMockService.prototype.generateId = /**
     * @private
     * @param {?=} min
     * @param {?=} max
     * @return {?}
     */
    function (min, max) {
        if (min === void 0) { min = 1; }
        if (max === void 0) { max = 100000; }
        return Math.floor(Math.random() * (max - min + 1) + min);
    };
    /**
     * @param {?} branchId
     * @param {?} metricId
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getMetricChange = /**
     * @param {?} branchId
     * @param {?} metricId
     * @param {?} tokenDetails
     * @return {?}
     */
    function (branchId, metricId, tokenDetails) {
        var e_1, _a;
        /** @type {?} */
        var ret = [];
        try {
            for (var _b = __values(SmdbApiMockService_1._VERSION_SAMPLES), _c = _b.next(); !_c.done; _c = _b.next()) {
                var version = _c.value;
                ret.push({ version: version, value: parseFloat((Math.random() * 10).toFixed(2)) });
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return of(ret).pipe(this.delayService.getDelay());
    };
    /**
     * @param {?} branchId
     * @param {?} startDate
     * @param {?} endDate
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getDeveloper = /**
     * @param {?} branchId
     * @param {?} startDate
     * @param {?} endDate
     * @param {?} tokenDetails
     * @return {?}
     */
    function (branchId, startDate, endDate, tokenDetails) {
        return of((/** @type {?} */ ([
            {
                developerStat: {
                    averageAddedLines: 20,
                    averageModifiedLines: 12,
                    averageRemovedLines: 3,
                    averageRuleViolationsCreated: 1,
                    averageRuleViolationsRemoved: 5,
                    maintainabilityChange: 2,
                    numberOfCommits: 24,
                    totalAddedLines: 8,
                    totalModifiedLines: 5,
                    totalRemovedLines: 0,
                    totalRuleViolationsCreated: 124,
                    totalRuleViolationsRemoved: 11,
                },
                name: 'John Doe',
            },
            {
                developerStat: {
                    averageAddedLines: 40,
                    averageModifiedLines: 16,
                    averageRemovedLines: 5,
                    averageRuleViolationsCreated: 0,
                    averageRuleViolationsRemoved: 12,
                    maintainabilityChange: 1.5,
                    numberOfCommits: 2.9,
                    totalAddedLines: 68,
                    totalModifiedLines: 35,
                    totalRemovedLines: 4,
                    totalRuleViolationsCreated: 42,
                    totalRuleViolationsRemoved: 16,
                },
                name: 'Little Joe',
            },
            {
                developerStat: {
                    averageAddedLines: 11,
                    averageModifiedLines: 32,
                    averageRemovedLines: 3,
                    averageRuleViolationsCreated: 0.3,
                    averageRuleViolationsRemoved: 5,
                    maintainabilityChange: -0.5,
                    numberOfCommits: 4,
                    totalAddedLines: 25,
                    totalModifiedLines: 51,
                    totalRemovedLines: 9,
                    totalRuleViolationsCreated: 14,
                    totalRuleViolationsRemoved: 11,
                },
                name: 'Sarah',
            },
            {
                developerStat: {
                    averageAddedLines: 30,
                    averageModifiedLines: 19,
                    averageRemovedLines: 8,
                    averageRuleViolationsCreated: 7,
                    averageRuleViolationsRemoved: 4.5,
                    maintainabilityChange: 5,
                    numberOfCommits: 24,
                    totalAddedLines: 8,
                    totalModifiedLines: 5,
                    totalRemovedLines: 2,
                    totalRuleViolationsCreated: 124,
                    totalRuleViolationsRemoved: 22,
                },
                name: 'Richard',
            },
            {
                developerStat: {
                    averageAddedLines: 52,
                    averageModifiedLines: 12,
                    averageRemovedLines: 3,
                    averageRuleViolationsCreated: 0,
                    averageRuleViolationsRemoved: 6.5,
                    maintainabilityChange: 0.8,
                    numberOfCommits: 15,
                    totalAddedLines: 80,
                    totalModifiedLines: 7,
                    totalRemovedLines: 10,
                    totalRuleViolationsCreated: 73,
                    totalRuleViolationsRemoved: 31,
                },
                name: 'Markus',
            },
        ]))).pipe(this.delayService.getDelay());
    };
    /**
     * @param {?} version
     * @param {?=} sourceCodeElementId
     * @param {?=} tokenDetails
     * @param {?=} priority
     * @param {?=} warningType
     * @return {?}
     */
    SmdbApiMockService.prototype.getWarnings = /**
     * @param {?} version
     * @param {?=} sourceCodeElementId
     * @param {?=} tokenDetails
     * @param {?=} priority
     * @param {?=} warningType
     * @return {?}
     */
    function (version, sourceCodeElementId, tokenDetails, priority, warningType) {
        /** @type {?} */
        var result = [];
        /** @type {?} */
        var n = sourceCodeElementId ? 15 : 55;
        /** @type {?} */
        var MAX_COL = 30;
        /** @type {?} */
        var MAX_LINE = 90;
        /** @type {?} */
        var SKIP = 16;
        /** @type {?} */
        var genLineInfo = (/**
         * @return {?}
         */
        function () {
            /** @type {?} */
            var col = Math.floor(Math.random() * MAX_COL) + 1;
            /** @type {?} */
            var endCol = col + Math.floor(Math.random() * (MAX_COL - col));
            /** @type {?} */
            var line = Math.floor(Math.random() * (MAX_LINE - SKIP)) + SKIP;
            /** @type {?} */
            var endLine = line + Math.floor(Math.random() * (MAX_LINE - line));
            /** @type {?} */
            var path = "path/to/file" + Math.floor(Math.random() * 10);
            return { col: col, endCol: endCol, line: line, endLine: endLine, path: path };
        });
        for (var i = 0; i < n; i++) {
            /** @type {?} */
            var date = Date.now();
            /** @type {?} */
            var warn = {
                startDate: new Date(date - i * 10000000).toISOString(),
                priority: "P" + Math.floor(Math.random() * 5),
                type: "type " + Math.floor(Math.random() * 10),
                description: 'Short Description',
                lineInfo: genLineInfo(),
            };
            if ((!warningType || warningType === warn.type) &&
                (!priority || priority === warn.priority)) {
                result.push(warn);
            }
        }
        return of(result).pipe(this.delayService.getDelay());
    };
    /**
     * @param {?} sourceCodeElementId
     * @param {?} branchId
     * @param {?} metricId
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getSourceCodeElementMetricChange = /**
     * @param {?} sourceCodeElementId
     * @param {?} branchId
     * @param {?} metricId
     * @param {?} tokenDetails
     * @return {?}
     */
    function (sourceCodeElementId, branchId, metricId, tokenDetails) {
        var e_2, _a;
        /** @type {?} */
        var ret = [];
        try {
            for (var _b = __values(SmdbApiMockService_1._VERSION_SAMPLES), _c = _b.next(); !_c.done; _c = _b.next()) {
                var version = _c.value;
                ret.push({ version: version, value: parseFloat((Math.random() * 10).toFixed(2)) });
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return of(ret).pipe(this.delayService.getDelay());
    };
    /**
     * @param {?} sourceCodeElementId
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getMetricsForSourceCodeElementInVersion = /**
     * @param {?} sourceCodeElementId
     * @param {?} tokenDetails
     * @return {?}
     */
    function (sourceCodeElementId, tokenDetails) {
        return of({
            package: 'SourceMeter',
            metricId: 656,
            name: 'TLLOC',
            description: 'Total Logical Lines of Code',
            displayname: 'Total Logical Lines of Code',
            // tslint:disable-next-line:max-line-length
            longDescription: '<b>Method:</b> number of non-empty and non-comment code lines of the method, including the non-empty and non-comment lines of its anonymous and local classes.<br><b>Class:</b> number of non-empty and non-comment code lines of the class, including the non-empty and non-comment code lines of its local methods, anonymous, local, and nested classes.<br><b>Package:</b> number of non-empty and non-comment code lines of the package, including its subpackages.<br><b>Component:</b> number of non-empty and non-comment code lines of the component, including its subcomponents.',
            value: 128581,
        }, {
            package: 'SourceMeter',
            metricId: 616,
            name: 'TNM',
            description: 'Total Number of Methods',
            displayname: 'Total Number of Methods',
            // tslint:disable-next-line:max-line-length
            longDescription: '<b>Class:</b> number of methods in the class, including the inherited ones, as well as the inherited and local methods of its nested, anonymous, and local classes. Methods that override abstract methods are not counted.<br><b>Package:</b> number of methods in the package, including the methods of its subpackages.<br><b>Component:</b> number of methods in the component, including the methods of its subcomponents.',
            value: 16493,
        }, {
            package: 'SourceMeter',
            metricId: 138,
            name: 'CC',
            description: 'Clone Coverage',
            displayname: 'Clone Coverage',
            // tslint:disable-next-line:max-line-length
            longDescription: '<b>Method/class/package:</b> ratio of code covered by code duplications in the source code element to the size of the source code element, expressed in terms of the number of syntactic entities (statements, expressions, etc.).<br><b>Component:</b> ratio of code covered by code duplications in the component to the size of the component, expressed in terms of the number of syntactic entities (statements, expressions, etc.).',
            value: 0.1966537,
        }, {
            package: 'SourceMeter',
            metricId: 490,
            name: 'CCL',
            description: 'Clone Classes',
            displayname: 'Clone Classes',
            // tslint:disable-next-line:max-line-length
            longDescription: '<b>Method/class/package:</b> number of clone classes having at least one clone instance in the source code element.<br><b>Component:</b> number of clone classes having at least one clone instance in the component.',
            value: 715,
        }, {
            package: 'SourceMeter',
            metricId: 143,
            name: 'CI',
            description: 'Clone Instances',
            displayname: 'Clone Instances',
            // tslint:disable-next-line:max-line-length
            longDescription: '<b>Method/class/package:</b> number of clone instances in the source code element.<br><b>Component:</b> number of clone instances in the component.<br><b>Clone class:</b> number of clone instances in the clone class.',
            value: 1868,
        }, {
            package: 'SourceMeter',
            metricId: 607,
            name: 'WarningMinor',
            description: 'Minor Warnings',
            displayname: 'Minor Warnings',
            longDescription: 'Number of warnings with Minor priority',
            value: 15730,
        }, {
            package: 'SourceMeter',
            metricId: 164,
            name: 'WarningBlocker',
            description: 'Blocker Warnings',
            displayname: 'Blocker Warnings',
            longDescription: 'Number of warnings with Blocker priority',
            value: 0,
        }, {
            package: 'SourceMeter',
            metricId: 596,
            name: 'WarningInfo',
            description: 'Info Warnings',
            displayname: 'Info Warnings',
            longDescription: 'Number of warnings with Info priority',
            value: 55583,
        }, {
            package: 'SourceMeter',
            metricId: 404,
            name: 'WarningMajor',
            description: 'Major Warnings',
            displayname: 'Major Warnings',
            longDescription: 'Number of warnings with Major priority',
            value: 2549,
        }).pipe(this.delayService.getDelay());
    };
    /**
     * @return {?}
     */
    SmdbApiMockService.prototype.getStatisticDetails = /**
     * @return {?}
     */
    function () {
        return of({
            branches: 3275,
            metricDefinitions: 2157,
            metrics: 599055139,
            projects: 3265,
            sces: 10574312,
            versions: 417277,
        }).pipe(this.delayService.getDelay());
    };
    /**
     * @param {?} projectId
     * @param {?} sourceCodeElementId
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getSourceCodeElementDetails = /**
     * @param {?} projectId
     * @param {?} sourceCodeElementId
     * @param {?} tokenDetails
     * @return {?}
     */
    function (projectId, sourceCodeElementId, tokenDetails) {
        return of({
            fromDate: new Date().toDateString(),
            projectName: 'Sample project',
            sourceCodeElementId: Math.floor(Math.random() * 100),
            toDate: new Date().toDateString(),
            type: 'attribute',
            uniqueName: 'com.alibaba.fastjson.JSON.DEFAULT_GENERATE_FEATURE;',
        });
    };
    /**
     * @param {?} projectName
     * @param {?} branchName
     * @param {?=} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getBranchIdByProjectNameAndBranchName = /**
     * @param {?} projectName
     * @param {?} branchName
     * @param {?=} tokenDetails
     * @return {?}
     */
    function (projectName, branchName, tokenDetails) {
        return of({ branchId: Math.floor(Math.random() * 100) });
    };
    /**
     * @param {?} versionId
     * @param {?} sourceCodeElementId
     * @return {?}
     */
    SmdbApiMockService.prototype.getSceLineInfo = /**
     * @param {?} versionId
     * @param {?} sourceCodeElementId
     * @return {?}
     */
    function (versionId, sourceCodeElementId) {
        /** @type {?} */
        var sce = {
            line: 10,
            endLine: 50,
            col: 5,
            endCol: 5,
            path: 'guava/src/com/google/common/base/AbstractIterator.java',
            sourceCodeElementId: 1,
            type: 'Method',
            uniqueName: 'com.google.common.base.AbstractIterator.hasNext()',
        };
        return of(sce).pipe(this.delayService.getDelay());
    };
    /**
     * @param {?} versionId
     * @param {?} path
     * @param {?=} branchId
     * @param {?=} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getSourceCode = /**
     * @param {?} versionId
     * @param {?} path
     * @param {?=} branchId
     * @param {?=} tokenDetails
     * @return {?}
     */
    function (versionId, path, branchId, tokenDetails) {
        return of("\n/*\n  * Copyright (C) 2007 The Guava Authors\n  *\n  * Licensed under the Apache License, Version 2.0 (the \"License\"); you may not use this file except\n  * in compliance with the License. You may obtain a copy of the License at\n  *\n  * http://www.apache.org/licenses/LICENSE-2.0\n  *\n  * Unless required by applicable law or agreed to in writing, software distributed under the License\n  * is distributed on an \"AS IS\" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express\n  * or implied. See the License for the specific language governing permissions and limitations under\n  * the License.\n  */\n\npackage com.google.common.base;\n\nimport static com.google.common.base.Preconditions.checkState;\n\nimport com.google.common.annotations.GwtCompatible;\nimport com.google.errorprone.annotations.CanIgnoreReturnValue;\nimport java.util.Iterator;\nimport java.util.NoSuchElementException;\nimport org.checkerframework.checker.nullness.qual.Nullable;\n\n/**\n * Note this class is a copy of {@link com.google.common.collect.AbstractIterator} (for dependency\n * reasons).\n */\n@GwtCompatible\nabstract class AbstractIterator<T> implements Iterator<T> {\n  private State state = State.NOT_READY;\n\n  protected AbstractIterator() {}\n\n  private enum State {\n    READY,\n    NOT_READY,\n    DONE,\n    FAILED,\n  }\n\n  private @Nullable T next;\n\n  protected abstract T computeNext();\n\n  @CanIgnoreReturnValue\n  protected final @Nullable T endOfData() {\n    state = State.DONE;\n    return null;\n  }\n\n  @Override\n  public final boolean hasNext() {\n    checkState(state != State.FAILED);\n    switch (state) {\n      case READY:\n        return true;\n      case DONE:\n        return false;\n      default:\n    }\n    return tryToComputeNext();\n  }\n\n  private boolean tryToComputeNext() {\n    state = State.FAILED; // temporary pessimism\n    next = computeNext();\n    if (state != State.DONE) {\n      state = State.READY;\n      return true;\n    }\n    return false;\n  }\n\n  @Override\n  public final T next() {\n    if (!hasNext()) {\n      throw new NoSuchElementException();\n    }\n    state = State.NOT_READY;\n    T result = next;\n    next = null;\n    return result;\n  }\n\n  @Override\n  public final void remove() {\n    throw new UnsupportedOperationException();\n  }\n}\n    ").pipe(this.delayService.getDelay());
    };
    /**
     * @param {?} versionId
     * @param {?} params
     * @return {?}
     */
    SmdbApiMockService.prototype.getSourceCodeQualifications = /**
     * @param {?} versionId
     * @param {?} params
     * @return {?}
     */
    function (versionId, params) {
        /** @type {?} */
        var ret = [
            {
                uniqueName: 'com.nostra13.universalimageloader.core.ImageLoader.displayImage(Ljava/lang/String;Landroid/widget/ImageView;Lcom/nostra13/universalimageloader/core/DisplayImageOptions;Lcom/nostra13/universalimageloader/core/ImageLoadingListener;)V',
                type: 'Sajt',
                qualifications: {
                    'Code complexity': 0.5062034739454092,
                    'Code size': 0.0512820512820511,
                    Maintainability: 0.31665288116901,
                },
            },
            {
                uniqueName: 'com.nostra13.universalimageloader.core.ImageLoader.getImageSizeScaleTo(Landroid/widget/ImageView;)Lcom/nostra13/universalimageloader/core/ImageSize;',
                type: 'Method',
                qualifications: {
                    'Code complexity': 1.2737799834574026,
                    'Code size': 0.1025641025641022,
                    Maintainability: 0.7857733664185275,
                },
            },
            {
                uniqueName: 'com.nostra13.universalimageloader.core.ImageDecoder.computeImageScale(Ljava/io/InputStream;)I',
                type: 'Method',
                qualifications: {
                    'Code complexity': 1.2390405293631097,
                    'Code size': 0.2051282051282055,
                    Maintainability: 0.8082437275985663,
                },
            },
            {
                uniqueName: 'com.nostra13.universalimageloader.cache.memory.LimitedMemoryCache.put(Ljava/lang/Object;Ljava/lang/Object;)Z',
                type: 'Method',
                qualifications: {
                    'Code complexity': 0.9842845326716297,
                    'Code size': 0.6153846153846154,
                    Maintainability: 0.8305762338020405,
                },
            },
            {
                uniqueName: 'com.nostra13.universalimageloader.core.ImageLoader$DisplayImageTask.loadBitmap()Landroid/graphics/Bitmap;',
                type: 'Method',
                qualifications: {
                    'Code complexity': 1.6526054590570718,
                    'Code size': 0.1538461538461533,
                    Maintainability: 1.028122415219189,
                },
            },
            {
                uniqueName: 'com.nostra13.example.universalimageloader.ImageListActivity$ItemAdapter.getView(ILandroid/view/View;Landroid/view/ViewGroup;)Landroid/view/View;',
                type: 'Method',
                qualifications: {
                    'Code complexity': 1.7038875103391236,
                    'Code size': 0.2564102564102566,
                    Maintainability: 1.1007719878687623,
                },
            },
            {
                uniqueName: 'com.nostra13.universalimageloader.cache.memory.FuzzyKeyMemoryCache.put(Ljava/lang/Object;Ljava/lang/Object;)Z',
                type: 'Method',
                qualifications: {
                    'Code complexity': 1.416046319272126,
                    'Code size': 1.0256410256410253,
                    Maintainability: 1.253377446925834,
                },
            },
            {
                uniqueName: 'com.nostra13.universalimageloader.cache.disc.LimitedDiscCache.put(Ljava/lang/String;Ljava/io/File;)V',
                type: 'Method',
                qualifications: {
                    'Code complexity': 1.4822167080231603,
                    'Code size': 1.0256410256410253,
                    Maintainability: 1.2919768403639373,
                },
            },
            {
                uniqueName: 'com.nostra13.universalimageloader.utils.StorageUtils.getExternalCacheDir(Landroid/content/Context;)Ljava/io/File;',
                type: 'Method',
                qualifications: {
                    'Code complexity': 1.6211745244003306,
                    'Code size': 0.871794871794872,
                    Maintainability: 1.3089330024813894,
                },
            },
            {
                uniqueName: 'com.nostra13.universalimageloader.cache.disc.LimitedDiscCache.removeNext()J',
                type: 'Method',
                qualifications: {
                    'Code complexity': 1.9172870140612075,
                    'Code size': 0.5128205128205132,
                    Maintainability: 1.3320926385442515,
                },
            },
            {
                uniqueName: 'com.nostra13.universalimageloader.cache.disc.LimitedDiscCache.put(Ljava/lang/String;Ljava/io/File;)VV',
                type: 'Method',
                qualifications: {
                    'Code complexity': 0.4822167080231603,
                    'Code size': 0.7256410256410253,
                    Maintainability: 4.8919768403639373,
                },
            },
            {
                uniqueName: 'com.nostra13.universalimageloader.utils.StorageUtils.getExternalCacheDir(Landroid/content/Context;)Ljava/io/demo/File;',
                type: 'Demo',
                qualifications: {
                    'Code complexity': 2.6211745244003306,
                    'Code size': 1.871794871794872,
                    Maintainability: 2.3089330024813894,
                },
            },
            {
                uniqueName: 'com.nostra13.universalimageloader.cache.disc.LimitedDiscCache.removeNext()VJ',
                type: 'Method',
                qualifications: {
                    'Code complexity': 3.9172870140612075,
                    'Code size': 0.15128205128205132,
                    Maintainability: 4.3320926385442515,
                    'Fourth property': 888.6666,
                },
            },
            {
                uniqueName: 'com.nostra13.universalimageloader.cache.disc.LimitedDiscCache.removeNext()VJ',
                type: 'Method',
                qualifications: {
                    'Code complexity': 3.9172870140612075,
                    'Code size': 0.15128205128205132,
                    'Fourth property': 9999.6666,
                    Maintainability: 4.3320926385442515,
                },
            },
        ];
        if (params.order &&
            params.order.length > 0 &&
            params.qualityAttribute &&
            params.qualityAttribute.length > 0) {
            /** @type {?} */
            var keys_1 = [];
            /** @type {?} */
            var dir = params.order === 'asc';
            /** @type {?} */
            var factor_2 = dir ? 1 : -1;
            ret.map((/**
             * @param {?} csce
             * @return {?}
             */
            function (csce) {
                Object.keys(csce.qualifications).map((/**
                 * @param {?} key
                 * @return {?}
                 */
                function (key) {
                    if (keys_1.indexOf(key) === -1) {
                        keys_1.push(key);
                    }
                }));
            }));
            /** @type {?} */
            var lowerCaseNames = keys_1.map((/**
             * @param {?} name
             * @return {?}
             */
            function (name) { return name.toLowerCase().replace(' ', ''); }));
            /** @type {?} */
            var lowerCase = lowerCaseNames.filter((/**
             * @param {?} name
             * @return {?}
             */
            function (name) { return name === params.qualityAttribute.toLowerCase().replace(' ', ''); }));
            /** @type {?} */
            var index_1 = lowerCaseNames.indexOf(lowerCase[0]);
            ret = ret.sort((/**
             * @param {?} a
             * @param {?} b
             * @return {?}
             */
            function (a, b) { return factor_2 * Math.sign(b.qualifications[keys_1[index_1]] - a.qualifications[keys_1[index_1]]); }));
        }
        /** @type {?} */
        var startIndex = params.size * params.page;
        /** @type {?} */
        var endIndex = params.size * (params.page + 1);
        return of({
            content: ret.slice(startIndex, endIndex),
            pageable: {
                sort: {
                    sorted: false,
                    unsorted: true,
                },
                offset: 0,
                pageSize: 10,
                pageNumber: 0,
                paged: true,
                unpaged: false,
            },
            totalPages: 2,
            totalElements: ret.length,
            last: false,
            number: 0,
            size: 10,
            sort: {
                sorted: false,
                unsorted: true,
            },
            first: true,
            numberOfElements: 10,
        });
    };
    /**
     * @param {?} projectId
     * @param {?} tokenDetails
     * @return {?}
     */
    SmdbApiMockService.prototype.getProjectDetails = /**
     * @param {?} projectId
     * @param {?} tokenDetails
     * @return {?}
     */
    function (projectId, tokenDetails) {
        return of({
            analysisRunning: Math.random() < 0.2,
            language: 'java',
            id: projectId,
            projectName: "Sample project " + projectId,
            url: 'https://github.com/' + projectId + '_th_project',
            deleted: false,
            private: !!this.generateId(0, 1),
        });
    };
    var SmdbApiMockService_1;
    SmdbApiMockService._VERSION_SAMPLES = [
        {
            author: 'John Doe',
            committed: '2019-04-30T14:45:47.048Z',
            hash: '1234567891234556345353',
            id: 48,
            log: 'Sample commit log message a long one',
            parentHash: '21534544354324234234234324',
        },
        {
            author: 'Jane Doe',
            committed: '2018-04-30T14:45:47.048Z',
            hash: '123456fsdfsdf91234556345353',
            id: 15,
            log: 'Another Sample commit log message',
            parentHash: '2143434544354324234234234324',
        },
        {
            author: 'John Doe',
            committed: '2019-01-30T14:45:47.048Z',
            hash: '1234567891234556345353',
            id: 16,
            log: 'Sample commit log message a long one',
            parentHash: '21534544354324234234234324',
        },
        {
            author: 'Jane Doe',
            committed: '2018-08-01T14:45:47.048Z',
            hash: '123456fsdfsdf91234556345353',
            id: 18,
            log: 'Another Sample commit log message',
            parentHash: '2143434544354324234234234324',
        },
    ];
    SmdbApiMockService._SAMPLE_BRANCH = {
        analysisRetryAttempts: 0,
        archived: false,
        branchName: 'master',
        deleted: false,
        language: 'java',
        lastAnalyzed: new Date().toDateString(),
        projectName: 'Sample project',
        status: 'ANALYSING',
        url: 'https://github.com/google/guava.git',
        versionsLeft: 5,
        id: 42,
        analyzerMessage: '',
        lastVersion: SmdbApiMockService_1._VERSION_SAMPLES[0],
        analyzedVersions: 412,
    };
    SmdbApiMockService._SAMPLE_BRANCH2 = {
        analysisRetryAttempts: 2,
        archived: false,
        branchName: 'develop',
        deleted: false,
        language: 'java',
        lastAnalyzed: new Date().toDateString(),
        projectName: 'Sample project',
        status: 'ANALYSIS_FAILED',
        url: 'https://github.com/google/guava.git',
        versionsLeft: 5,
        id: 27,
        analyzerMessage: 'Unknown exception',
        lastVersion: SmdbApiMockService_1._VERSION_SAMPLES[1],
        analyzedVersions: 12,
    };
    SmdbApiMockService._SAMPLE_BRANCH3 = {
        analysisRetryAttempts: 3,
        archived: false,
        branchName: 'trunk',
        deleted: false,
        language: 'java',
        lastAnalyzed: new Date().toDateString(),
        projectName: 'Sample project',
        status: 'ANALYSIS_FAILED',
        url: 'https://github.com/google/guava.git',
        versionsLeft: 5,
        id: 36,
        analyzerMessage: 'Another exception',
        lastVersion: SmdbApiMockService_1._VERSION_SAMPLES[2],
        analyzedVersions: 105,
    };
    SmdbApiMockService._BRANCHES = [
        SmdbApiMockService_1._SAMPLE_BRANCH,
        SmdbApiMockService_1._SAMPLE_BRANCH2,
        SmdbApiMockService_1._SAMPLE_BRANCH3,
    ];
    SmdbApiMockService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    SmdbApiMockService.ctorParameters = function () { return [
        { type: DelayService }
    ]; };
    // @dynamic
    SmdbApiMockService = SmdbApiMockService_1 = __decorate([
        MethodLoggerDecorator(['constructor', 'generateId']),
        __metadata("design:paramtypes", [DelayService])
    ], SmdbApiMockService);
    return SmdbApiMockService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var SmdbApiMockModule = /** @class */ (function () {
    function SmdbApiMockModule() {
    }
    /**
     * @return {?}
     */
    SmdbApiMockModule.forRoot = /**
     * @return {?}
     */
    function () {
        return {
            ngModule: SmdbApiMockModule,
            providers: [
                DelayService,
                { provide: SmdbApiService, useClass: SmdbApiMockService, deps: [DelayService] },
            ],
        };
    };
    SmdbApiMockModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [],
                    imports: [CommonModule],
                    providers: [],
                    exports: [],
                },] }
    ];
    return SmdbApiMockModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { SmdbApiMockService, DelayService, SmdbApiMockModule, MethodLoggerDecorator as ɵa };

//# sourceMappingURL=frontendart-smdb-api-mock.js.map