import { Observable } from 'rxjs';
import { TokenDetails, BranchDTO, ProjectDTO, VersionDetails, VersionDTO, QualificationResult, Page, MetricDefinition, MetricVersionMap, Developer, WarningResult, Stat, MetricValue, SourceCodeElementWithMetricValues, SourceCodeQualificationDTO, SourceCodeElementDetails, SourceCodeElement } from '@frontendart/smdb-common';
import { DelayService } from './delay.service';
export declare class SmdbApiMockService {
    private delayService;
    constructor(delayService: DelayService);
    private static _VERSION_SAMPLES;
    private static _SAMPLE_BRANCH;
    private static _SAMPLE_BRANCH2;
    private static _SAMPLE_BRANCH3;
    private static _BRANCHES;
    private providedSampleBranches;
    checkToken(token: string): Observable<TokenDetails>;
    getDeveloperStatistics(branchId: number, tokenDetails: TokenDetails): Observable<Developer[]>;
    getBranchMetric(branch: number, tokenDetails: TokenDetails): Observable<MetricDefinition[]>;
    getBranchDetails(branch: number, tokenDetails: TokenDetails): Observable<BranchDTO>;
    getVersionDetails(versionId: number, tokenDetails: TokenDetails): Observable<VersionDetails>;
    findProjects(projectName: string, pageSize: number, page: number, privateOnly: boolean, tokenDetails: TokenDetails): Observable<Page<ProjectDTO>>;
    getBranches(projectId: number, tokenDetails: TokenDetails): Observable<BranchDTO[]>;
    setProvidedSampleBranches: BranchDTO[];
    static readonly SAMPLE_BRANCH: BranchDTO;
    getTimeline(projectName: string, branchName: string, tokenDetails: TokenDetails): Observable<string>;
    getBranchVersionData(branch: number, tokenDetails: TokenDetails): Observable<VersionDTO[]>;
    getQualifications(type: string, id: number, tokenDetails: TokenDetails): Observable<QualificationResult[]>;
    /**
     * Mocks get /api/version/{versionId}/sourceCodeElementsByMetric future version
     */
    getSourceCodeElementsWithMetrics(versionId: number, params: {
        uniqueName?: string;
        metricDefinitionId: number;
        size?: number;
        type?: string;
        page?: number;
        sort?: string;
    }, tokenDetails?: TokenDetails): Observable<Page<SourceCodeElementWithMetricValues>>;
    private generateId;
    getMetricChange(branchId: number, metricId: number, tokenDetails: TokenDetails): Observable<MetricVersionMap[]>;
    getDeveloper(branchId: number, startDate: string, endDate: string, tokenDetails: TokenDetails): Observable<Developer[]>;
    getWarnings(version: number, sourceCodeElementId?: number, tokenDetails?: TokenDetails, priority?: string, warningType?: string): Observable<WarningResult[]>;
    getSourceCodeElementMetricChange(sourceCodeElementId: number, branchId: number, metricId: number, tokenDetails: TokenDetails): Observable<MetricVersionMap[]>;
    getMetricsForSourceCodeElementInVersion(sourceCodeElementId: number, tokenDetails: TokenDetails): Observable<MetricValue>;
    getStatisticDetails(): Observable<Stat>;
    getSourceCodeElementDetails(projectId: number, sourceCodeElementId: number, tokenDetails: TokenDetails): Observable<SourceCodeElementDetails>;
    getBranchIdByProjectNameAndBranchName(projectName: string, branchName: string, tokenDetails?: TokenDetails): Observable<{
        branchId: number;
    }>;
    getSceLineInfo(versionId: number, sourceCodeElementId: number): Observable<SourceCodeElement>;
    getSourceCode(versionId: number, path: string, branchId?: number, tokenDetails?: TokenDetails): Observable<string>;
    getSourceCodeQualifications(versionId: number, params: {
        model?: string;
        qualityAttribute?: string;
        order?: string;
        size?: number;
        page?: number;
        token?: TokenDetails;
    }): Observable<Page<SourceCodeQualificationDTO>>;
    getProjectDetails(projectId: number, tokenDetails: TokenDetails): Observable<ProjectDTO>;
}
